__doc__='''Shopper product implementation'''

__version__='$Revision: 1.4 $'[11:-2]

ShopperError = "Shopper Error."

from Globals import HTMLFile, MessageDialog, Persistent
import OFS.SimpleItem, Acquisition, AccessControl.Role
from Products import ZPatterns
import string
import time
from Products import ZPatterns

class ShopperManager( ZPatterns.Specialists.Specialist ):

    """ A Shopper Manager """

    # Specify a name for the item type:
    meta_type='ShopperManager'

    def newShopper(self, shopperID, passwd, email  ):
        """ Create a new Shopper... """

        shopper = self.newItem( shopperID )
        shopper.passwd = passwd
        shopper.email = email
        shopper.transCount = 1

        # this shopper needs a new basket...
        
        newBasket = self.basketManager.newShoppingBasket( shopperID )

        return shopper

    def currentBasketId( self, shopperId):
        """ get the current basket id.. """
        shopper = self.getItem(shopperId)
        return shopperId + '_' + `shopper.transCount`
    
    def addToBasket(self, REQUEST=None, RESPONSE=None):
        """ Add and item to the basket... get everything from the 'REQUEST'

        returns  statusMsg, statusCode

        where

            statusMsg = "OK" if all is well and offer is high enough
                        "Failed" if there is some unrecoverable problem

            statusCode =
                       -1 cannot find any MarketItem to match request
                       1  attempt to drop quantity
                       2  not enough product remaining
                       3  attempt to drop offer
                       4  offer is below minPrice, or it's an auction"""
    
        itemID = REQUEST.get('itemID',None)
        quantity = REQUEST.get('quantity',0)
        updateBasket = REQUEST.get('updateBasket',0)
        shopperId = REQUEST.get('clientCookie')
        basketId = self.currentBasketId( shopperId )

        if type(quantity) == type(''):
            quantity = string.strip(quantity)
            quantity = string.replace(quantity,',','')
            quantity = int(string.atoi(quantity))

        optionDict={}

        if REQUEST.has_key('optionDict'):
            optionDict = REQUEST['optionDict']

        elif REQUEST.has_key('options'):
            options = string.split(string.strip(REQUEST['options'])[1:-1],',')
            for option in options:
                option = string.strip(option)[1:-1]  # get rid of quotes!
                optionDict[option] = REQUEST.get(option,'')
        
        item = self.marketFolder.getItem(itemID)

        if item is not None: # we found the item....
            if quantity > 0:
                self.basketManager.addItemToBasket( shopperId, item, quantity, updateBasket, REQUEST )
            else:
                self.basketManager.deleteItemFromBasket( basketId, item, REQUEST )

            returnVal = "OK",0
        else:
            returnVal="Failed",-1

        return returnVal

    def clearPastTransactions(self):
        """ Clear out the past transactions of a shopper.."""
        self.basket.clearProcessedItems()
        return "OK!"
    
    def addSalesItem(self, REQUEST=None):
        """ Add and item for sale... get everything from the 'REQUEST' """

        if REQUEST is None:
            return "Hello? not much to do......"
        
        quantityAvailable = REQUEST.get('quantity',1)
        itemID = REQUEST.get('itemID')
        sku = REQUEST.get('sku')
        updateBooth=REQUEST.get('updateBooth',None)
        enabled=REQUEST.get('enabled',0)

        price = REQUEST.get('price')

        if type(price) == type(''):
            price = string.strip(price)
            price = string.replace(price,',','')
            price = string.replace(price, '$','')
            try:
                price = string.atof(price)
            except:
                price = None

        minPrice = REQUEST.get('minPrice')

        if type(minPrice) == type(''):
            minPrice = string.strip(minPrice)
            minPrice = string.replace(minPrice,',','')
            minPrice = string.replace(minPrice,'$','')
            try:
                minPrice = string.atof(minPrice)
            except:
                minPrice = None

        item = None

        item= self.getMarketItem(sku)

        if item is not None:
            item.quantityAvailable=quantityAvailable

            if price is not None:
                item.price = price
            else:
                item.price = 0.0

            if minPrice is not None:
                item.minPrice = minPrice
            else:
                item.minPrice = 0.0

            item.enabled = enabled
            
            self.salesItems.addItem( anItem=item, updateBooth=updateBooth )
            return "OK","No problem..."
        else:
            return "Failed","Hmm can't find that market item..."

    def getSalesItems(self):
        """ return the active sales items for a seller...."""

        theItems = self.salesItems.items
        result = []

        for k,v in theItems.items():
            if self.getMarketItem(v.sku) == None:
                del theItems[k]  # get rid of items that have been deleted....
            else:
                result.append(v)
        return result

    def loginAsUser(self, REQUEST=None, RESPONSE=None, PARENTS=None, PATH_INFO=None):
        """ set the browser cookie to user id for this user..."""

        result = self.set_cookie(RESPONSE = RESPONSE, REQUEST=REQUEST,
                                 path=PATH_INFO, parents=PARENTS, clientCookie=self.id)

        if REQUEST is not None:
            result = MessageDialog(
                title='Logged In',
                message='You are logged in as <strong>%s</strong><br> %s.' % (self.id, result),
                action = "./manage_main")
        else:
            result = "OK"

        return result

    def setProcessed(self, transID):
        """ check transID for current-status..."""

        result = 0
        
        if transID == self.transID:
            result=1
            self.basket.processTransaction(transID)

        return result
    
    def manage_changePass(self, oldpass, passwd, passwd2, origURL=None, REQUEST=None, passExcept=0):
        """ Change the password. """

        try:

            result = "OK"

            if oldpass != self.passwd:
                raise ShopperError, "Bad password."

            if len(passwd) < 5:
                raise ShopperError, "Password must be longer than 4 characters..."

            if passwd != passwd2:
                raise ShopperError, "Passwords must match..."

            self.passwd = passwd

            if REQUEST is not None:
                if origURL is None:
                    origURL = './manage_main'

                result = MessageDialog(
                    title='Edited',
                    message='<strong>%s</strong>\'s password has been edited.' % self.id,
                    action = origURL
                    )

        except ShopperError, msg:
            result = "<html><body bgcolor=ffffff><center><h3>Sorry %s</h3></center></body></html>" % msg
            if passExcept:
                raise ShopperError, result

        return result

    def manage_self_edit(self, email, name, address, city, state, zip, country='', company='',
                         phone='', fax='', origURL=None, REQUEST=None):
        """
        Allow user to manage themselves.....

        """
        #

        if REQUEST.has_key('clientCookie') and (REQUEST['clientCookie'] == self.id):
            result = self.manage_edit(email, name, address, city, state, zip, country, phone=phone, fax=fax,
                                      company=company, origURL=origURL, REQUEST=REQUEST)

            if REQUEST.has_key('passwd') and REQUEST['passwd']:
                self.manage_changePass(oldpass = self.passwd,
                                       passwd = REQUEST.get('passwd',''),
                                       passwd2 = REQUEST.get('passwd2',''),
                                       passExcept = 1)
        else:
            result = "Sorry.... you are not logged in as this shopper...."

        return result
    
    def manage_edit(self, email, name, address, city, state, zip, country='', company='',
                    phone='', fax='', passwd='', passwd2='', origURL=None, REQUEST=None):
	"""
	Edit a shopper
	"""
        try:
            if '@' not in email:
                raise ShopperError, "Email must contain an '@'."
            else:
                if passwd == '':
                    passwd = self.passwd
                    passwd2 = self.passwd

                if len(passwd) < 5:
                    raise ShopperError, "Password must be longer than 4 characters..."

                if passwd != passwd2:
                    raise ShopperError, "Passwords must match..."

                self.passwd = passwd
                self.email = email
                self.name = name
                self.address = address
                self.city = city
                self.state = state
                self.zip = zip
                self.country = country
                self.company = company
                self.phone = phone
                self.fax = fax

                if REQUEST is not None:
                    if origURL is None:
                        origURL = './manage_main'

                    result = MessageDialog(
                        title='Edited',
                        message='<strong>%s</strong> has been edited.' % self.id,
                        action = origURL
                        )
                        
        except ShopperError, msg:
            result = "Sorry %s " % msg

        return result




